#!/bin/bash

readonly CURRENT_DIR=`/usr/bin/readlink -f $0 | xargs /usr/bin/dirname`
source ${CURRENT_DIR}/../edge.env
readonly NETSCRIPTS_DIR="/etc/sysconfig/network-scripts/"
readonly MOD_FILE="/etc/modprobe.conf"
readonly MOD_DIR="/etc/modprobe.d"
readonly ROUTE_SCRIPT="$EDGE_HOME/bin/admin-cfg-scripts/network/addroute.sh"
readonly IPTABLES_FILE="/etc/sysconfig/iptables"
readonly TMP_DIR="$EDGE_BASE/tmp"
readonly ECHO=/bin/echo
readonly BOND_PREFIX="bond"

checkPermission()
{
    if [ $UID -gt 0 ] && [ "`id -un`" != "root" ]; then
	${ECHO} "This script can be run by root only!"
	exit 1
    fi
}

checkAddParams()
{
    echo $1 | grep "^${BOND_PREFIX}.*$"
    if [ ! $? -eq 0 ]; then
	${ECHO} "Ileagal bonding interface name, must start with '${BOND_PREFIX}'"
    	exit 1
    fi
}

function clear_all_interfaces()
{
    # STOP the network service before resetting ifcfg-*
    # If network is stopped after modifying ifcfg-*, network stop may fail due to ifdown-eth reads ifcfg-* firstly
    /sbin/service network stop
    TMP_MODFILE="$MOD_FILE.tmp"
    cat $MOD_FILE | sed "/^.*bond.*$/d">$TMP_MODFILE
    cp -f $TMP_MODFILE $MOD_FILE
    rm -rf $TMP_MODFILE
    rm -rf $NETSCRIPTS_DIR/ifcfg-${BOND_PREFIX}*
    find $MOD_DIR -name "${BOND_PREFIX}*.conf" -exec rm -rf {} \;
    find $NETSCRIPTS_DIR -name "ifcfg-*" | grep -v ifcfg-lo | xargs rm -rf

    # Re-generate ifcfg-eth*
    # Retrieve slave interface from bonding files
    local SLAVE_INTERFACE=(`ls /proc/net/bonding/* | sort | xargs grep "Slave Interface:" | awk -F": " '{print $2}'`)
    local SLAVE_HWADDR=(`ls /proc/net/bonding/* | sort | xargs grep "Permanent HW addr:" | awk -F": " '{print toupper($2)}'`)
    for ((i=0; i<${#SLAVE_INTERFACE[@]}; i++))
    do
        ${ECHO} "interface is ${SLAVE_INTERFACE[$i]}"
        ${ECHO} "hwaddr is ${SLAVE_HWADDR[$i]}"
        ${ECHO} -e "DEVICE=${SLAVE_INTERFACE[$i]}\nHWADDR=${SLAVE_HWADDR[$i]}\nONBOOT=yes">"$NETSCRIPTS_DIR/ifcfg-${SLAVE_INTERFACE[$i]}"
    done
    local IFS=(`/sbin/ifconfig -a |grep eth|awk -F " " '{print $1}'`)
    local HW_ADDR
    for IF in ${IFS[@]}
    do
        if [ ! -f "$NETSCRIPTS_DIR/ifcfg-${IF}" ]; then
            ${ECHO} "interface is ${IF}"
            HW_ADDR=`/sbin/ifconfig ${IF}| grep HWaddr | awk -F "HWaddr" '{print $2}' | sed s/[[:space:]]//g`
            ${ECHO} "hwaddr is "${HW_ADDR}
            ${ECHO} -e "DEVICE=${IF}\nHWADDR=${HW_ADDR}\nONBOOT=yes">"$NETSCRIPTS_DIR/ifcfg-${IF}"
        fi
    done
}

printUsage()
{
    ${ECHO} "Usage:"
    ${ECHO} -e "$0 \n-clean\n-add interfaces_names bond_name bond_ip bond_netmask bond_gateway\n-add interfaces_names bond_name\n-backupcfg backup_index\n-restorecfg backup_index\n-deletecfg backup_index"
    ${ECHO} "Example: $0 -add eth0,eth1,eth2 bond0 172.21.105.68 255.255.255.0 172.21.105.254"
    ${ECHO} "Example: $0 -add eth0,eth1,eth2 bond0"
    ${ECHO} "Example: $0 -clean"
    ${ECHO} "Example: $0 -backupcfg 20120802"
    ${ECHO} "Example: $0 -restorecfg 20120802"
    ${ECHO} "Example: $0 -deletecfg 20120802"
    ${ECHO} "NOTE -- \"$0 -clean\" will stop network service"
}

checkPermission

case "$1" in
    -clean)
        ${ECHO} "cleaning bond interfaces and restoring the old configuration files..."
        clear_all_interfaces
	;;
    -add)
        if [ $# != 3 -a $# != 6 ]; then
            ${ECHO} "parameter error"
            printUsage
            exit 1
        fi
        checkAddParams $3
        ${ECHO} "adding bond interface..."
        BONDNAME=$3

	    ${ECHO} "updating ifcfg-$BONDNAME file..."
	    BONDCFG_FILE="$NETSCRIPTS_DIR""ifcfg-""$BONDNAME"
        if [ $# = 6 ]; then
            BONDIP=$4
            BONDMASK=$5
            BONDGW=$6
            ${ECHO} -e "DEVICE=$BONDNAME\nIPADDR=$BONDIP\nNETMASK=$BONDMASK\nGATEWAY=$BONDGW\nUSERCTL=no\nBOOTPROTO=none\nONBOOT=yes\n">$BONDCFG_FILE
        else
            ${ECHO} -e "DEVICE=$BONDNAME\nUSERCTL=no\nBOOTPROTO=none\nONBOOT=yes\n">$BONDCFG_FILE
        fi
	    if [ $? -ne 0 ]; then
            exit 1
        fi

        var=`$ECHO "$2" | sed "s/,/ /g "`
        for interface in $var
        do
            ${ECHO} "updating ifcfg-$interface file..."
            IFCFG_FILE="$NETSCRIPTS_DIR/ifcfg-$interface"
            TMP_IFCFG_FILE="$IFCFG_FILE.tmp"
            BAK_IFCFG_FILE="$TMP_DIR/ifcfg-$interface.rpadbak"
            if [ ! -f $BAK_IFCFG_FILE ]; then
                cp -f $IFCFG_FILE $BAK_IFCFG_FILE
            fi
            ${ECHO} -e "DEVICE=$interface\nUSERCTL=no\nONBOOT=yes\nMASTER=$BONDNAME\nSLAVE=yes\nBOOTPROTO=none\n">$TMP_IFCFG_FILE
            cp -f $TMP_IFCFG_FILE $IFCFG_FILE
            if [ $? -ne 0 ]; then
                exit 1
            fi
            rm -rf $TMP_IFCFG_FILE
        done
        
        ${ECHO} "updating $MOD_FILE file..."
	    TMP_MODFILE="$MOD_FILE.tmp"
        cp -f $MOD_FILE $TMP_MODFILE  
        ${ECHO} -e "alias $BONDNAME bonding\noptions $BONDNAME miimon=100 mode=0 maxbonds=">>$TMP_MODFILE
        if [ $? -ne 0 ]; then
            exit 1
        fi
        MAXBONDS=`grep -c bonding $TMP_MODFILE`
        cat $TMP_MODFILE | sed "s/maxbonds=*/maxbonds=$MAXBONDS/g">$MOD_FILE
        if [ $? -ne 0 ]; then
            exit 1
        fi
	    rm -rf $TMP_MODFILE
        echo "alias $BONDNAME bonding" >$MOD_DIR/"$BONDNAME.conf"
        /sbin/modprobe bonding
    ;;
    -backupcfg)
	    if [ -z $2 ]; then
            ${ECHO} "You must provice a backup/restore index."
            exit 1    
        fi
	    
        BAKDIR=$TMP_DIR/$2
	    ${ECHO} "backing up configuration files to $BAKDIR..."
        mkdir $BAKDIR
        var=`find $NETSCRIPTS_DIR -name "ifcfg-*" -exec basename {} \;`
        for cfgfile in $var
        do
        cp -f $NETSCRIPTS_DIR/$cfgfile $BAKDIR/$cfgfile
        if [ $? -ne 0 ]; then
            exit 1
        fi
        done
        cp -f $MOD_FILE $BAKDIR/modprobe.conf
        cp -f $ROUTE_SCRIPT $BAKDIR/addroute.sh
        cp -f $IPTABLES_FILE $BAKDIR/iptables
    ;;
    -restorecfg)
        RESDIR=$TMP_DIR/$2
        if [ ! -d $RESDIR ]; then
    	    ${ECHO} "restore directory: $RESDIR doesn't exist."
    	    exit 1 
        fi
        ${ECHO} "restoring configuration files from $RESDIR..."
        clear_all_interfaces
        RESDIR=$TMP_DIR/$2
        cd $RESDIR
        cp -f $RESDIR/ifcfg-* $NETSCRIPTS_DIR
        cp -f $RESDIR/modprobe.conf $MOD_FILE
        cp -f $RESDIR/addroute.sh $ROUTE_SCRIPT
        cp -f $RESDIR/iptables $IPTABLES_FILE
        /sbin/service network start
    ;; 
    -deletecfg)
        ${ECHO} "deleting backup $2..."
        DELDIR=$TMP_DIR/$2
        if [ -d $RESDIR ]; then
    	    rm -rf $DELDIR 
        fi   
    ;;
    *)
        ${ECHO} "parameter error"
        printUsage
        exit 1
    ;;
esac

${ECHO} "You need to reboot the machine to make the changes done!"
